/* K575IO.c 
 * lowlevel hardware communication to K575
 * Written by Matthias D. Tews 1995, 1996 
 * 
 */

#include <K575.h>
#include <linux/sched.h>	/* just only for "jiffies" */

/*** interface to memory mapped IO */
#define __KPTR ((unsigned char *)0xECF00)	/* where to find the K575 */
#define K575_mem __KPTR

void
SetByte (unsigned off, unsigned data)
{
  K575_mem[off] = data;
}

unsigned
GetByte (unsigned off)
{
  return K575_mem[off];
}

static void
CMDA (unsigned SLOT, unsigned DATA)
{
  SetByte (0x80 + 2 * (SLOT), DATA);
}

static void
CMDB (unsigned SLOT, unsigned DATA)
{
  SetByte (0x81 + 2 * (SLOT), DATA);
}

static unsigned
RCMDA (unsigned SLOT)
{
  return GetByte (0x80 + 2 * (SLOT));
}

static unsigned
RCMDB (unsigned SLOT)
{
  return GetByte (0x81 + 2 * (SLOT));
}

/***********************************************/
/*** analog output with AOM1 */

void
AOM1_SetVoltage (unsigned SLOT, unsigned CHANNEL, unsigned value)
{
  CMDA (SLOT, 2 * CHANNEL);
  CMDB (SLOT, value % 256);

  CMDA (SLOT, 2 * CHANNEL + 1);
  CMDB (SLOT, value / 256);
}

void
AOM1_Out ()
{
  SetByte (0x9D, 1);
}

void
AOM1_Init ()
{
  SetByte (0x9D, 64);
}

/***********************************************/
/*** digital in/out with K575 motherboard */

#define DSLOT 4

static unsigned DIO_SELECT = 128;

void
DIO_SelectOut (unsigned channel)
{
  DIO_SELECT |= (1u << (channel + 3));
  CMDA (DSLOT, DIO_SELECT + channel);
  CMDB (DSLOT, 0);
}

void
DIO_SelectIn (unsigned channel)
{
  DIO_SELECT &= ~(1u << (channel + 3));
  CMDA (DSLOT, DIO_SELECT);
}

void
DIO_SetByte (unsigned channel, unsigned v)
{
  CMDA (DSLOT, channel);
  CMDB (DSLOT, v);
}

unsigned
DIO_GetByte (unsigned channel)
{
  CMDA (DSLOT, channel);
  return RCMDB (DSLOT);
}

void
DIO_SetBit (unsigned bit)
{
  unsigned channel;

  channel = bit / 8u;
  bit = 1u << (bit % 8u);
  CMDA (DSLOT, channel);
  CMDB (DSLOT, RCMDB (DSLOT) | bit);
}

void
DIO_UnsetBit (unsigned bit)
{
  unsigned channel;

  channel = bit / 8u;
  bit = 1u << (bit % 8u);
  CMDA (DSLOT, channel);
  CMDB (DSLOT, RCMDB (DSLOT) & ~bit);
}

unsigned
DIO_GetBit (unsigned bit)
{
  unsigned channel;

  channel = bit / 8u;
  bit = 1u << (bit % 8u);
  CMDA (DSLOT, channel);
  bit &= RCMDB (DSLOT);
  if (bit)
    return 1;
  return 0;
}

#undef DSLOT

/***********************************************/
/*** Analog Input with AMM1A Module */

static void
AMM1A_StartConversion ()
{
  SetByte (0x9B, 0);
}

int
AMM1A_EndOfConversion ()
{
  return !(GetByte (0x9B) >> 7);
}

void
AMM1A_Convert (int channel, int gain, int filter)
/* possible values of gain: 1 2 5 10 20 50 100 */
{
  unsigned a, b;

  a = 0x10;			/* single ended input */
  b = 0x11;			/* select hardware input from slot 1 */

  a |= channel;

  if (filter == 2)
    a |= 0x80;			/* select 2kHz, default= 100kHz */

  if (gain < 0)
    {
      b |= 0x20;		/* set range to -10 to +10, default
				   is 0 to +10 */
      gain = -gain;
    }
  if (gain > 10)
    {
      gain /= 10;
      a |= 0x20;		/* local gain = x10 */
    }
  switch (gain)
    {				/* select global gain    */
    case 1:
      break;
    case 2:
      b |= 0x40;
      break;
    case 5:
      b |= 0x80;
      break;
    case 10:
      b |= 0xC0;
      break;
    }

  CMDB (0, b);
  CMDA (0, a);

  AMM1A_StartConversion ();
}

void
AMM1A_ConvertDiff (int channel, int gain, int filter)
/* possible values of gain: 1 2 5 10 20 50 100 */
{
  unsigned a, b;

  a = 0x00;			/* differential input */
  b = 0x11;			/* select hardware input from slot 1 */

  a |= channel;

  if (filter == 2)
    a |= 0x80;			/* select 2kHz, default= 100kHz */

  if (gain < 0)
    {
      b |= 0x20;		/* set range to -10 to +10, default
				   is 0 to +10 */
      gain = -gain;
    }
  if (gain > 10)
    {
      gain /= 10;
      a |= 0x20;		/* local gain = x10 */
    }
  switch (gain)
    {				/* select global gain    */
    case 1:
      break;
    case 2:
      b |= 0x40;
      break;
    case 5:
      b |= 0x80;
      break;
    case 10:
      b |= 0xC0;
      break;
    }

  CMDB (0, b);
  CMDA (0, a);

  AMM1A_StartConversion ();
}

unsigned
AMM1A_value ()
{
  return RCMDA (0) | (RCMDB (0) << 8);
}

void
AMM1A_Init ()
{
  int jiff;

  jiff = jiffies + 61;
  CMDA (1, 0);			/* NO TRIGGER */
  SetByte (0x9A, 0);
  while (jiff >= jiffies);	/* wait 0.6 seconds -- :) */
}

/***********************************************/
/*** timer and interrupts */

/* we need an alternative interface to memory mapped io, since
   caching destroys our datas */
static void
Poke (unsigned offset, unsigned data)
{
  SetByte (offset, data);
  schedule ();
}

static unsigned
Peak (unsigned offset)
{
  schedule ();
  return GetByte (offset);
}

/* no we shall to do only "clean" things */
void
CNT_SetCounter (unsigned counter, unsigned value)
{
  unsigned v;

  v = counter << 6;
  v |= 0x34;
  SetByte (0xC3, v);
  Poke (0xC0 + counter, value % 256);
  Poke (0xC0 + counter, value / 256);
}

unsigned
CNT_GetCounter (int counter)
{
  unsigned v;

  v = counter << 6;
  v |= 0x04;
  SetByte (0xC3, v);
  return Peak (0xC0 + counter) + 256 * Peak (0xC0 + counter);
}

void
CNT_SetIrqLevel (int irq)
{
  Poke (0xE3, 15);
  switch (irq)
    {
    case 2:
    case 9:
      Poke (0xE3, 15 - 1);
      break;
    case 3:
      Poke (0xE3, 15 - 2);
      break;
    case 5:
      Poke (0xE3, 15 - 4);
      break;
    default:
      Poke (0xE0, 0x03);
      return;
    }
}

void
CNT_EndOfInterrupt ()
{
  SetByte (0xE2, 0);
}

void
CNT_Init ()
{
  Poke (0xE0, 0x00);
  CNT_SetCounter (0, 1);
  CNT_SetCounter (1, 1);
  CNT_SetCounter (2, 1);
}
