/* k575lib.c
 * user interface to K575
 *
 * written by Matthias D. Tews
 * 
 */

#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <fcntl.h>
#include <k575lib.h>
#include <k575.h>

static int K575_fd = -1;

static void
CheckErr (int __errno, char *func)
{
  if (__errno == 0)
    return;
  __errno = errno;

  fprintf (stderr, "K575 error in %s\r\n", func);
  if (K575_fd == -1)
    fprintf (stderr, "K575 not opened\r\n");
  fprintf (stderr, "error status: %s\r\n", strerror (__errno));

  exit (3);
}

void
K575_SelectOut (int i)
{
  CheckErr (ioctl (K575_fd, K575_DIO_SelectOut, &i), "K575_SelectOut");
}

void
K575_SelectIn (int i)
{
  CheckErr (ioctl (K575_fd, K575_DIO_SelectIn, &i), "K575_SelectIn");
}

void
K575_open ()
{
  int r;

  fprintf (stderr, "%s",
	   "K575 interface libary\r\n"
	   "2.0 - written by M.Tews\r\n\r\n");

  K575_fd = open ("/dev/K575", O_RDWR | O_SYNC);

  CheckErr (K575_fd == -1, "K575_open");

  K575_SelectOut (1);
  K575_SelectIn (0);
  for (r = 0; r < 5; r++)
    K575_SetAnalog (r, 2 * 1024);
  K575_OutAnalog ();
  fprintf (stderr, "K575 opened\n");
}

void
K575_close ()
{
  if (K575_fd == -1)
    return;

  /* try it 2 times */
  if (close (K575_fd) == 0)
    goto exit;
  if (close (K575_fd) == 0)
    goto exit;
  fprintf (stderr, "cannot close K575");
  return;
exit:fprintf (stderr, "K575 closed\n");
}

void
K575_SetBit (int i)
{
  CheckErr (ioctl (K575_fd, K575_DIO_SetBit, &i), "K575_SetBit");
}

void
K575_UnsetBit (int i)
{
  CheckErr (ioctl (K575_fd, K575_DIO_UnsetBit, &i), "K575_UnsetBit");
}

int
K575_GetBit (int i)
{
  CheckErr (ioctl (K575_fd, K575_DIO_GetBit, &i), "K575_SetBit");
  return i;
}

int
K575_GetByte (int i)
{
  CheckErr (ioctl (K575_fd, K575_DIO_GetByte, &i), "K575_GetByte");
  return i;
}

void
K575_SetByte (int channel, unsigned v)
{
  v |= channel << 16;
  CheckErr (ioctl (K575_fd, K575_DIO_SetByte, &v), "K575_SetByte");
}

void
K575_SetAnalog (int nr, unsigned v)
{
  v |= nr << 16;
  CheckErr (ioctl (K575_fd, K575_AOM1_SetVoltage, &v), "K575_SetAnalog");
}

void
K575_OutAnalog ()
{
  CheckErr (ioctl (K575_fd, K575_AOM1_Out, 0), "K575_OutAnalog");
}

void
K575_WriteAnalog (int nr, unsigned v)
{
  K575_SetAnalog (nr, v);
  K575_OutAnalog ();
}

void
K575_Convert (int channel, int gain, int filter)
{
  K575_ConvertStruct buf;

  buf.channel = channel;
  buf.gain = gain;
  buf.filter = filter;

  CheckErr (ioctl (K575_fd, K575_AMM1A_Convert, &buf), "K575_Convert");
}

void
K575_ConvertDiff (int channel, int gain, int filter)
{
  K575_ConvertStruct buf;

  buf.channel = channel;
  buf.gain = gain;
  buf.filter = filter;

  CheckErr (ioctl (K575_fd, K575_AMM1A_ConvertDiff, &buf), "K575_Convert");
}

unsigned
K575_GetConvertedValue ()
{
  unsigned buf;

  CheckErr (ioctl (K575_fd, K575_AMM1A_Value, &buf), "K575_GetConvertedValue");
  return buf;
}

unsigned
K575_IsConversionReady ()
{
  unsigned buf;

  CheckErr (ioctl (K575_fd, K575_AMM1A_Ready, &buf), "K575_IsConversionReady");

  return buf;
}

unsigned
K575_ReadAnalog (int channel, int gain, int filter)
{
  int i;

  K575_Convert (channel, gain, filter);
  for (i = 0; i < 40000; i++)
    if (K575_IsConversionReady ())
      return K575_GetConvertedValue ();

  fprintf (stderr, "K575 error in %s\r\n", "K575_ReadAnalog");
  if (K575_fd == -1)
    fprintf (stderr, "K575 not opened\r\n");
  fprintf (stderr, "K575 power off !!! \r\n");
  exit (3);

  return 0;
}

unsigned
K575_ReadAnalogDiff (int channel, int gain, int filter)
{
  int i;

  K575_ConvertDiff (channel, gain, filter);
  for (i = 0; i < 40000; i++)
    if (K575_IsConversionReady ())
      return K575_GetConvertedValue ();

  fprintf (stderr, "K575 error in %s\r\n", "K575_ReadAnalogDiff");
  if (K575_fd == -1)
    fprintf (stderr, "K575 not opened\r\n");
  fprintf (stderr, "K575 power off !!! \r\n");
  exit (3);
  return 0;
}

static struct timeval timebase;

void
K575_StartTimer (void)
{
  gettimeofday (&timebase, NULL);
}

long
K575_Time_ms (void)
{
  long d_secs;
  long d_musecs;
  struct timeval diff;

  gettimeofday (&diff, NULL);
  d_secs = diff.tv_sec - timebase.tv_sec;
  d_musecs = diff.tv_usec - timebase.tv_usec;
  return d_secs * 1000 + d_musecs / 1000;
}


#if 0
/* very daft timer function */

void
start ()
{
  gettimeofday (&timebase, NULL);
}

void
period (struct timeval *diff)
{
  gettimeofday (diff, NULL);

  if (diff->tv_usec < timebase.tv_usec)
    {
      diff->tv_usec += 1000000;
      diff->tv_sec--;
    }
  diff->tv_usec -= timebase.tv_usec;
  diff->tv_sec -= timebase.tv_sec;
}
#endif


void K575_Write_mV (int channel, int voltage_mV)
{
  K575_WriteAnalog (channel, ((voltage_mV + 10000) * 4095 / 20000) );
}

int K575_Read_mV (int channel, int gain, int filter)
{
  return ( K575_ReadAnalog(channel, gain, filter) * 20000 / 65520 - 10000 );
}


/* may be used be "root" only */
void
K575_Poke (unsigned offset, unsigned data)
{
  data |= offset << 16;
  CheckErr (ioctl (K575_fd, K575_POKE, &data), "K575_POKE");
}

/* may be used be "root" only */
unsigned
K575_Peak (unsigned offset)
{
  offset = offset << 16;
  CheckErr (ioctl (K575_fd, K575_PEAK, &offset), "K575_PEAK");
  return offset;
}

unsigned
K575_GetCounter (unsigned nr)
{
  CheckErr (ioctl (K575_fd, K575_CNT_GetCounter, &nr), "K575_GetCounter");
  return nr;
}

void
K575_SetCounter (unsigned nr, unsigned value)
{
  value |= nr << 16;
  CheckErr (ioctl (K575_fd, K575_CNT_SetCounter, &value), "K575_SetCounter");
}

void
K575_SetInterruptMode (unsigned mode)
{
  CheckErr (ioctl (K575_fd, K575_CNT_InterruptMode, &mode),
	    "K575_SetInterruptMode");
}
